import platform
import subprocess
from datetime import datetime
import os

import qrcode
from string import Template
from weasyprint import HTML
import markdown as markdown


def open_file(filepath):
    if platform.system() == 'Darwin':       # macOS
        subprocess.Popen(['open', filepath], start_new_session=True)
    elif platform.system() == 'Windows':    # Windows
        os.startfile(filepath)
    else:                                   # linux variants
        subprocess.Popen(['xdg-open', filepath], start_new_session=True)


def generate_paper_wallet(cyphertext, output_file='output/paperKey.pdf', open_pdf=False, title=None, notes=None):
    """
    generates a pdf containing a QR Code with the cyphertext, the cyphertext as text and explanations on how to decrypt
    :param open: open the file in the default program
    :param cyphertext:
    :param output_file: if True opens the file in your default pdf program
    :return:
    """
    now = datetime.now()  # generate timestamp for file generation and to show in generated pdf

    if output_file is None:
        output_file = f'paperKey_{now.strftime("%Y%m%d%H%M%S")}.pdf'

    #  NamedTempFile has issues in windows, as does using an absolute directory and referencing it in markdown.
    #  Therefore we handle creation and deletion of the file ourselves
    qr_tmp_filename = f"{os.urandom(24).hex()}.png"

    try:
        with open(qr_tmp_filename, 'wb') as fp:
            img = qrcode.make(cyphertext).resize((250, 250))
            img.save(fp)

            d = {
                'cyphertext': cyphertext,
                'aesMode': 'AES-256 CBC PBKDF2',
                'qrCodeFile': fp.name,
                'generationDateTime': now.strftime("%m/%d/%Y, %H:%M:%S"),
                'title': f"### {title} " if title else '###',
                'notes': f"#### Notes\n{notes}" if notes else ''
            }
            with open(os.path.join(os.path.dirname(__file__), 'templates/printTemplate.md'), 'r') as templateFile:
                with open(os.path.join(os.path.dirname(__file__), 'templates/styles.css'), mode="r", encoding="utf-8") as css_file:

                    src = Template(templateFile.read())
                    result = src.substitute(d)
                    input_html = markdown.markdown(
                        result, extensions=["markdown.extensions.abbr",
                                            "markdown.extensions.attr_list",
                                            "markdown.extensions.def_list",
                                            "markdown.extensions.fenced_code",
                                            "markdown.extensions.footnotes",
                                            "markdown.extensions.tables",
                                            "markdown.extensions.md_in_html"]
                    )
                    css_input = css_file.read()

                    output_html = f"""
                        <html>
                            <head>
                                <style>{css_input}</style>
                            </head>
                            <body class="markdown-body">{input_html}</body>
                        </html>
                        """
                    html = HTML(string=output_html, base_url='output')

                    html.write_pdf(output_file)
                    print("--------------------")
                    print(f"encrypted paper generated in\n file://{os.path.abspath(output_file)}")
                    if open_pdf:
                        open_file(os.path.abspath(output_file))

    finally:
        os.remove(qr_tmp_filename)



import hashlib

SALT_PREFIX = b'Salted__'
SALT_LENGTH_BYTES = 8


def pbkdf2_derive_key_iv(password, salt, show_key=False):
    """
    derives key and iv according to AES256-CBC
    :param show_key: log derived key and iv if set to true
    :param password:
    :param salt:
    :return: aes_key, iv
    """
    derived_key = hashlib.pbkdf2_hmac('sha256', password, salt, 10000, 48)
    aes_key = derived_key[0:32]
    iv = derived_key[32:48]
    if show_key:
        print(f'key={aes_key.hex().upper()}\niv={iv.hex().upper()}\nlen={len(aes_key.hex())}')
    return aes_key, iv



from setuptools import setup, find_packages
from os import path
import codecs
import os.path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setup(name='pyaes256',
      version=get_version("pyaes256/__init__.py"),
      install_requires=['markdown', 'pillow', 'qrcode', 'weasyprint', 'cairocffi', 'pycryptodome'],
      author='Marko Nalis',
      author_email="marko@nalis.dev",
      url='https://github.com/The-Crocop/pyaes256',
      description="Encrypt text with AES256 and create a pdf with QR code that can be printed and stored",
      long_description=long_description,
      long_description_content_type='text/markdown',
      packages=find_packages(),
      package_data={
          'pyaes256': ['templates/*']
      },
      license='MIT',
      include_package_data=True,
      entry_points={
          'console_scripts': ['pyaes256=pyaes256.__main__:run']
      }
      )



#!/usr/bin/python

import argparse
import codecs
import os
from getpass import getpass

from pyaes256 import __version__
from pyaes256.decrypt import decrypt
from pyaes256.encrypt import encrypt


def validate_salt(value):
    if len(value) != 16:
        raise argparse.ArgumentTypeError("Salt must be a 16 characters long (hex)")
    return value


def get_version():
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, '__init__.py'), 'r') as fp:
        for line in fp.read().splitlines():
            if line.startswith('__version__'):
                delim = '"' if '"' in line else "'"
                return line.split(delim)[1]
        else:
            raise RuntimeError("Unable to find version string.")


def read_password(confirm=True):
    password = getpass()
    confirmed_password = getpass('Confirm Password:') if confirm else password
    if password != confirmed_password:
        raise Exception('Passwords do not match!')
    if len(password) < 1:
        raise Exception('Passwords is too short!')
    return password


def run():
    parser = argparse.ArgumentParser(
        description='Encrypt and decrypt text with AES-256 CBC and pbkdf2 as base64 like openssl')
    parser.add_argument('action', choices=['encrypt', 'decrypt'])
    parser.add_argument('input',
                        metavar='input',
                        type=str,
                        help='the text to encrypt/decrypt depending on the mode')

    parser.add_argument('-p', '--password',
                        metavar='password',
                        type=str,
                        help='the password')
    parser.add_argument('-o', '--output',
                        metavar='output',
                        type=str,
                        required=False,
                        help='target filename and path')
    parser.add_argument('-s', '--salt',
                        metavar='salt',
                        type=validate_salt,
                        required=False,
                        help='you can optionally specify a salt (in hex format) 16 characters')
    parser.add_argument('-sk', '--show-key',
                        help='shows the key and iv', action='store_const', const=True)
    parser.add_argument('--no-open',
                        help='do not automatically open the pdf', action='store_const', const=True)
    parser.add_argument('-t', '--title',
                        metavar='title',
                        type=str,
                        required=False,
                        help='optionally specify a title for the paper wallet')
    parser.add_argument('-n', '--notes',
                        metavar='notes',
                        type=str,
                        required=False,
                        help='optionally specify some notes that will be rendered in the notes section')
    parser.add_argument('--version', action='version', version=__version__)

    args = parser.parse_args()
    if 'encrypt' == args.action:
        if args.password is None:
            args.password = read_password()
        print('encrypting with aes256...')
        encrypted_text = encrypt(args.input, args.password.encode(), args.salt, show_key=args.show_key is not None)
        print(f"Encrypted: {encrypted_text}")
        from pyaes256.paper_wallet import generate_paper_wallet
        generate_paper_wallet(encrypted_text, output_file=args.output, open_pdf=args.no_open is None, title=args.title,
                              notes=args.notes)

    elif 'decrypt' == args.action:
        if args.password is None:
            args.password = read_password(confirm=False)
        decrypted_text = decrypt(args.input, args.password.encode(), show_key=args.show_key is not None)
        print(f"Decrypted: {decrypted_text}")


if __name__ == "__main__":
    run()



from pyaes256.decrypt import decrypt
from pyaes256.encrypt import encrypt


def test_encrypt_and_decrypt():
    # given:
    secret = 'young cement bubble network scout radio mask output maze alone balance seed party large couch forget' \
             ' paddle swarm person way yard switch open loop\n'
    password = b'abcdefg'
    # when:
    encrypted_text = encrypt(secret, password)
    print(encrypted_text)
    decrypted_text = decrypt(encrypted_text, password)
    print(decrypted_text)
    # then
    assert secret == decrypted_text


def test_decrypt_generated_by_openssl():
    # given:
    expected_output = 'young cement bubble network scout radio mask output maze alone balance seed party large couch' \
                      ' forget paddle swarm person way yard switch open loop\n'
    password = b'abcdefg'
    text_to_decrypt = 'U2FsdGVkX1/HEDOl4/ffIWPNVohxRpJleiPUp4VRKxkoBN2YUny/kFVh/3lp06+WAhnUAxsW5yBtGPZLV6e2314pgi' \
                      'FXbth4nEZ4xOtyNEHlYb3cBcjsDMrmZz4O41iN8lf+Hj6oKl7tD/nnQ5kMFHm1GE+1zDDJ0N0r/M2pvCuSAVXPFWD7' \
                      'F1NlsVjG9g9I8rz2/9CdpYiM8d7sAa0CeMNRauUqYegMY+X5NlB/rXY='
    # when:
    decrypted_text = decrypt(text_to_decrypt, password)
    # then:
    assert decrypted_text == expected_output
